package com.jeeplus.test.reimbursementsys.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.jeeplus.aop.demo.annotation.DemoMode;
import com.jeeplus.aop.logging.annotation.ApiLog;
import com.jeeplus.core.excel.utils.EasyPoiUtil;
import com.jeeplus.sys.utils.StringUtils;
import com.jeeplus.test.reimbursementsys.domain.dto.ReimbursementDTO;
import com.jeeplus.test.reimbursementsys.service.ReimbursementSysService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author li-peike
 * @version 1.0.0
 * @ClassName ReimbursementSysController.java
 * @Description TODO 发票信息
 * @createTime 2022年05月05日 20:41:00
 */
@RestController
@RequestMapping("/reimbursement")
public class ReimbursementSysController {

    @Autowired
    private ReimbursementSysService service;

    /**
     * 根据id查询部门
     * @param id
     * @return
     */
    @ApiLog("查询业务具体信息")
    @PreAuthorize("hasAnyAuthority('reimbursement:view','reimbursement:add','reimbursement:edit')")
    @GetMapping("queryBusinessById")
    public ResponseEntity queryBusinessById(@RequestParam String id) {
        ReimbursementDTO reimbursementDTO = service.queryBusinessById (id);
        return ResponseEntity.ok (reimbursementDTO);
    }
    /**
     * 根据id查询部门
     * @param id
     * @return
     */
    @ApiLog("查询发票具体信息")
    @PreAuthorize("hasAnyAuthority('reimbursement:view','reimbursement:add','reimbursement:edit')")
    @GetMapping("queryInvoiceById")
    public ResponseEntity queryInvoiceById(@RequestParam String id) {
        ReimbursementDTO reimbursementDTO = service.queryInvoiceById (id);
        return ResponseEntity.ok (reimbursementDTO);
    }


    /**
     * 查询报销信息
     * @return
     */
    @ApiLog("查询业务报销信息")
    @GetMapping("treeData")
    public ResponseEntity treeData(ReimbursementDTO reimbursementDTO,  Page<ReimbursementDTO> page) {
        IPage<ReimbursementDTO> list = service.treeData(page,reimbursementDTO);
        return ResponseEntity.ok(list);
    }

    /**
     * 上传文件
     * @return
     * @throws IOException
     */
    @ApiLog("上传文件")
    @RequestMapping("uploadFile")
    public Map<String,Object> uploadFile(HttpServletRequest request, MultipartFile file) {
        Map<String,Object> map = new HashMap<>();
        try {
            List <ReimbursementDTO> list = EasyPoiUtil.importExcel ( file, 0, 1, ReimbursementDTO.class );
            /*ImportExcel ei = new ImportExcel(file, 0, 0);
            List<ReimbursementDTO> list = ei.getDataList(ReimbursementDTO.class);*/
            map = service.save(list);
            return map;
        } catch (Exception e) {
            map.put("message", "文件上传失败。失败原因："+ e.getMessage());
            map.put("success", false);
            return map;
        }
    }


    /**
     * 保存业务报销编码
     * @return
     */
    @ApiLog("保存业务报销编码")
    @PostMapping("saveBusiness")
    public ResponseEntity saveBusiness(@Valid @RequestBody ReimbursementDTO reimbursementDTO) {
        if(StringUtils.isBlank(reimbursementDTO.getBusinessCode())){
            return ResponseEntity.badRequest ().body ("业务编码不可为空，请重新填写!");
        }
        service.saveBusiness(reimbursementDTO);
        return ResponseEntity.ok("保存业务编码" + reimbursementDTO.getBusinessCode() + "成功");
    }
    /**
     * 保存业务报销编码
     * @return
     */
    @ApiLog("保存业务报销比例")
    @PostMapping("updateReimbursementRatio")
    public ResponseEntity updateReimbursementRatio(@Valid @RequestBody ReimbursementDTO reimbursementDTO) {
        if(StringUtils.isBlank(reimbursementDTO.getReimbursementRatio())){
            return ResponseEntity.badRequest ().body ("报销比例不可为空，请重新填写!");
        }
        service.updateReimbursementRatio(reimbursementDTO);
        return ResponseEntity.ok("保存业务编码" + reimbursementDTO.getBusinessCode() + "成功");
    }


    /**
     * 保存发票收款时间
     * @return
     */
    @ApiLog("保存发票收款时间")
    @PostMapping("saveInvoice")
    public ResponseEntity saveInvoice(@Valid @RequestBody ReimbursementDTO reimbursementDTO) {
        service.saveInvoice(reimbursementDTO);
        return ResponseEntity.ok("保存发票收款时间成功");
    }


    /**
     * 保存发票收款时间
     * @return
     */
    @ApiLog("修改发票信息")
    @PostMapping("modifyInvoice")
    public ResponseEntity modifyInvoice(@Valid @RequestBody ReimbursementDTO reimbursementDTO) {
        service.modifyInvoice(reimbursementDTO);
        return ResponseEntity.ok("修改发票信息成功");
    }

    /**
     * 批量删除
     * @param ids
     * @return
     */
    @DemoMode
    @ApiLog("批量删除")
    @PreAuthorize("hasAuthority('reimbursement:del')")
    @DeleteMapping("delete")
    public ResponseEntity delete(String ids) {
        String idArray[] =ids.split(",");
        service.removeBusinessByIds ( Lists.newArrayList (idArray) );
        return ResponseEntity.ok ("删除成功！");
    }

    /**
     * 保存业务报销编码
     * @return
     */
    @ApiLog("保存收款时间信息")
    @PostMapping("saveGatheringTime")
    public ResponseEntity saveGatheringTime(@Valid @RequestBody ReimbursementDTO reimbursementDTO) {
        if(StringUtils.isBlank(reimbursementDTO.getId())){
            return ResponseEntity.badRequest ().body ("未选择发票信息!");
        }
        if(null == reimbursementDTO.getGatheringTime()){
            return ResponseEntity.badRequest ().body ("未选择收款时间!");
        }
        Map<String, Object> map = service.saveGatheringTime(reimbursementDTO);
        Boolean success = (Boolean) map.get("success");
        if(success){
            return ResponseEntity.ok("保存业务编码" + reimbursementDTO.getBusinessCode() + "成功");
        }
        return ResponseEntity.badRequest().body ( map.get("message"));
    }

    /**
     * 删除业务信息
     * @param ids
     * @return
     */
    @DemoMode
    @ApiLog("删除业务信息")
    @PreAuthorize("hasAuthority('reimbursement:del')")
    @DeleteMapping("deleteBusiness")
    public ResponseEntity deleteBusiness(String ids) {
        String idArray[] =ids.split(",");
        service.removeBusinessByIds ( Lists.newArrayList (idArray) );
        return ResponseEntity.ok ("删除成功！");
    }

    /**
     * 删除发票信息
     * @param ids
     * @return
     */
    @DemoMode
    @ApiLog("删除发票信息")
    @PreAuthorize("hasAuthority('reimbursement:del')")
    @DeleteMapping("deleteInvoice")
    public ResponseEntity deleteInvoice(String ids) {
        String idArray[] =ids.split(",");
        service.removeInvoiceByIds ( Lists.newArrayList (idArray) );
        return ResponseEntity.ok ("删除成功！");
    }
}
