package com.jeeplus.test.reimbursementsys.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.jeeplus.sys.utils.StringUtils;
import com.jeeplus.test.reimbursementsys.domain.dto.ReimbursementDTO;
import com.jeeplus.test.reimbursementsys.mapper.ReimbursementSysMapper;
import com.jeeplus.test.reimbursementsys.service.ReimbursementBusinessService;
import com.jeeplus.test.reimbursementsys.service.ReimbursementSysService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author li-peike
 * @version 1.0.0
 * @ClassName ReimbursementSysServiceImpl.java
 * @Description TODO
 * @createTime 2022年05月06日 10:38:00
 */
@Service
public class ReimbursementSysServiceImpl implements ReimbursementSysService {

    @Resource
    private ReimbursementSysMapper mapper;
    @Resource
    private ReimbursementBusinessService reimbursementBusinessService;


    @Override
    public ReimbursementDTO queryBusinessById(String id) {
        return mapper.queryBusinessById(id);
    }

    @Override
    public ReimbursementDTO queryInvoiceById(String id) {
        return mapper.queryInvoiceById(id);
    }


    @Override
    public IPage<ReimbursementDTO> treeData(Page<ReimbursementDTO> page, ReimbursementDTO reimbursement) {
        IPage<ReimbursementDTO> pageList = new Page();
        //如果筛选项含有发票号码，则需先查询对应的发票信息
        if(StringUtils.isNotBlank(reimbursement.getInvoiceNumber()) || StringUtils.isNotBlank(reimbursement.getProposer()) || StringUtils.isNotBlank(reimbursement.getPartner())){
            List<ReimbursementDTO> invoiceList =  mapper.getInvoiceList(reimbursement);
            //如果存在发票信息，则根据发票的父节点查询对应的业务信息
            if(invoiceList.size()>0){
                Set<String> idSet = new HashSet<String>();
                for (ReimbursementDTO  info : invoiceList) {
                    idSet.add(info.getParentId());
                }
                reimbursement.setIdList(new ArrayList<>(idSet));
                pageList = mapper.queryAllList(page,reimbursement);

                List<ReimbursementDTO> list =  pageList.getRecords();
                if(list.size()>0){
                    //将子节点信息进行分组
                    Map<String, List<ReimbursementDTO>> invoiceListMap = dataMessageDispose(invoiceList);
                    //将分组后的子节点信息通过遍历的方式添加到父节点中的children中
                    for (ReimbursementDTO info:list) {
                        for (String key : invoiceListMap.keySet()) {
                            if (info.getId().equals(key)) {
                                List<ReimbursementDTO> children = invoiceListMap.get(key);
                                info.setChildren(children);
                            }
                        }
                        ReimbursementDTO businessById = reimbursementBusinessService.getBusinessById(info.getId());
                        if(null != businessById){
                            info.setAllAlreadyReimbursementQuota(businessById.getAllAlreadyReimbursementQuota());
                            info.setCanReimbursementAmount(businessById.getCanReimbursementAmount());
                            info.setSurplusReimbursementAmount(businessById.getSurplusReimbursementAmount());
                        }
                    }
                }
            }else{
                return page;
            }
        }else{
            pageList = mapper.queryAllList(page,reimbursement);
            List<ReimbursementDTO> list =  pageList.getRecords();
            if(list.size()>0){
                //获取所有发票信息表中的数据
                List<String> parentIdList = Lists.newArrayList();
                for (ReimbursementDTO info:list) {
                    parentIdList.add(info.getId());
                }
                //根据查询的父节点信息查询对应的子节点信息
                List<ReimbursementDTO> invoiceList =  mapper.getInvoiceByParentIdList(parentIdList);
                if(invoiceList.size()>0){
                    //将子节点信息进行分组
                    Map<String, List<ReimbursementDTO>> invoiceListMap = dataMessageDispose(invoiceList);
                    //将分组后的子节点信息通过遍历的方式添加到父节点中的children中
                    for (ReimbursementDTO info:list) {
                        for (String key : invoiceListMap.keySet()) {
                            if (info.getId().equals(key)) {
                                List<ReimbursementDTO> children = invoiceListMap.get(key);
                                info.setChildren(children);
                            }
                        }
                        ReimbursementDTO businessById = reimbursementBusinessService.getBusinessById(info.getId());
                        if(null != businessById){
                            info.setAllAlreadyReimbursementQuota(businessById.getAllAlreadyReimbursementQuota());
                            info.setCanReimbursementAmount(businessById.getCanReimbursementAmount());
                            info.setSurplusReimbursementAmount(businessById.getSurplusReimbursementAmount());
                        }
                    }
                }
            }
        }

        return pageList;
    }

    /**
     * 文件数据分组
     * @param dataList
     * @return
     */
    public Map<String, List<ReimbursementDTO>> dataMessageDispose(List<ReimbursementDTO> dataList) {
        //所有有效数据均添加到list中，下边将list中的数据根据店号和收银员的id进行分割处理
        Map<String, List<ReimbursementDTO>> result = listGroup2Map(dataList);
        return result;
    }

    private Map<String, List<ReimbursementDTO>> listGroup2Map(List<ReimbursementDTO> list) {
        Map<String, List<ReimbursementDTO>> userSignSerialMap = new HashMap<>();
        if (CollectionUtils.isNotEmpty(list)) {
            // 分组
            for (ReimbursementDTO vo : list) {
                List<ReimbursementDTO> tempList = userSignSerialMap.get(vo.getParentId());
                /*如果取不到数据,那么直接new一个空的ArrayList**/
                if (tempList == null) {
                    tempList = new ArrayList<>();
                    tempList.add(vo);
                    userSignSerialMap.put(vo.getParentId(), tempList);
                }
                else {
                    /*某个vo之前已经存放过了,则直接追加数据到原来的List里**/
                    tempList.add(vo);
                }
            }
        }
        return userSignSerialMap;
    }

    /**
     * 保存
     * @param reimbursementList
     */
    @Override
    public Map<String,Object> save(List<ReimbursementDTO> reimbursementList) {
        Map<String,Object> map = new HashMap<>();
        Set<String> invoiceNumberSet = new HashSet<String>();
        for (ReimbursementDTO info : reimbursementList) {
            if(StringUtils.isBlank(info.getBusinessCode())){
                info.setBusinessCode("");
            }
            invoiceNumberSet.add(info.getInvoiceNumber());
        }
        //如果发票编号数量不等于总数量，则表示所传数据中存在重复的发票编号，则进行抛出
        if(invoiceNumberSet.size() != reimbursementList.size()){
            map.put("success", false);
            map.put("message", "数据中存在相同的发票号，请重新验证上传");
            return map;
        }
        //set转list
        List<String> invoiceNumberList = new ArrayList<String>(invoiceNumberSet);
        //查询发票号是否存在，若含有已存在发票号，则直接进行抛出
        List<String> oldInvoiceNumberList = mapper.selectOldInvoiceNumber(invoiceNumberList);
        if(oldInvoiceNumberList.size()>0){
            String oldInvoiceNumber = String.join(",", oldInvoiceNumberList);
            map.put("success", false);
            map.put("message", "发票号码：" + oldInvoiceNumber + " 已存在。无法再次上传");
            return map;
        }

        //业务编号对应的数据集合
        List<ReimbursementDTO> businessCodeInfoList = Lists.newArrayList();
        List<ReimbursementDTO> businessCodeNullList = Lists.newArrayList();
        List<String> businessCodeList = Lists.newArrayList();
        //循环遍历reimbursementList 整理出相同的业务编号（null或者空字符串除外），并将这些业务编号进行去重，保存到数据库
        Map<String, List<ReimbursementDTO>> businessCodeListMap = dataDisposeOnSaveByBusinessCode(reimbursementList);
        for (String key : businessCodeListMap.keySet()) {
            if("".equals(key)){
                List<ReimbursementDTO> infoList = businessCodeListMap.get(key);
                businessCodeNullList.addAll(infoList);
            }else{
                List<ReimbursementDTO> infoList = businessCodeListMap.get(key);
                businessCodeInfoList.add(infoList.get(0));
                businessCodeList.add(key);
            }
        }

        //查询非空的业务编号数据在数据库中是否存在，若存在，返回已存在的业务编号集合
        List<String> oldBusinessCodeList = mapper.selectOldBusinessCode(businessCodeList);
        if (oldBusinessCodeList.size()>0){
            //如果数据库已存在部分业务编号信息，则去除该部分的业务编号信息
            for (String oldBusinessCode: oldBusinessCodeList) {
                Iterator iterator = businessCodeInfoList.iterator();
                while (iterator.hasNext()) {
                    ReimbursementDTO data = (ReimbursementDTO) iterator.next();
                    if (oldBusinessCode.equals(data.getBusinessCode())){
                        iterator.remove();
                    }
                }
            }

        }

        //将空的业务编号 的信息添加到集合中
        businessCodeInfoList.addAll(businessCodeNullList);
        //对业务编号的数据进行新增保存
        for (ReimbursementDTO newBusinessCodeInfo : businessCodeInfoList) {
            newBusinessCodeInfo.preInsert();
            newBusinessCodeInfo.setParentId("0");
            newBusinessCodeInfo.setParentIds("0,");
        }
        mapper.insertBusinessCodeList(businessCodeInfoList);


        //业务编号对应的数据集合
        List<ReimbursementDTO> invoiceInfoList = Lists.newArrayList();

        for (ReimbursementDTO info : businessCodeInfoList) {
            if(StringUtils.isNotBlank(info.getBusinessCode())){
                //处理业务编号不为空的数据
                for (String key : businessCodeListMap.keySet()) {
                    if(info.getBusinessCode().equals(key)){
                        List<ReimbursementDTO> infoList = businessCodeListMap.get(key);
                        for (ReimbursementDTO childrenInfo : infoList) {
                            if(info.getBusinessCode().equals("2021-01768")){
                                System.out.println("");
                            }
                            childrenInfo.setParentId(info.getId());
                            String parentIds = "0," + info.getId() + ",";
                            childrenInfo.setParentIds(parentIds);
                            //childrenInfo.preInsert();
                            invoiceInfoList.add(childrenInfo);
                        }
                    }
                }

            }else{
                //处理业务编号为空的数据
                info.setParentId(info.getId());
                String parentIds = "0," + info.getId() + ",";
                info.setParentIds(parentIds);
                //info.preInsert();
                invoiceInfoList.add(info);
            }
        }
        for (ReimbursementDTO childrenInfo : invoiceInfoList) {
            childrenInfo.preInsert();
        }
        //新增
        mapper.insertInvoiceList(invoiceInfoList);

        map.put("success", true);
        map.put("message", "数据上传成功");
        return map;
    }

    @Override
    public void saveBusiness(ReimbursementDTO reimbursement) {
        //查询该业务编码在数据库中是否存在
        ReimbursementDTO infoByBusinessCode = mapper.getInfoByBusinessCode(reimbursement.getBusinessCode());
        if(null != infoByBusinessCode){
            //如果存在该业务编码 则将该业务编码对应的发票数据的父节点变更为已存在的业务编码的id信息
            //根据id 查询对应的发票信息
            List<String> parentIdList = Lists.newArrayList();
            parentIdList.add(reimbursement.getId());
            List<ReimbursementDTO> invoiceByParentIdList = mapper.getInvoiceByParentIdList(parentIdList);
            if(invoiceByParentIdList.size()>0){
                List<ReimbursementDTO> updateInvoiceData = Lists.newArrayList();
                for (ReimbursementDTO invoiceInfo: invoiceByParentIdList) {
                    invoiceInfo.setBusinessCode(infoByBusinessCode.getBusinessCode());
                    invoiceInfo.setParentId(infoByBusinessCode.getId());
                    invoiceInfo.setParentIds("0," + infoByBusinessCode.getId() + ",");
                    invoiceInfo.preUpdate();
                    updateInvoiceData.add(invoiceInfo);
                }
                mapper.updateInvoiceList(updateInvoiceData);
            }
            //删除该业务编码信息
            mapper.deleteBusinessById(reimbursement.getId());
        }else{
            //根据id查询对应发票信息
            List<String> parentIdList = Lists.newArrayList();
            parentIdList.add(reimbursement.getId());
            List<ReimbursementDTO> invoiceByParentIdList = mapper.getInvoiceByParentIdList(parentIdList);
            if(invoiceByParentIdList.size()>0){
                List<ReimbursementDTO> updateInvoiceData = Lists.newArrayList();
                for (ReimbursementDTO invoiceInfo: invoiceByParentIdList) {
                    invoiceInfo.setBusinessCode(reimbursement.getBusinessCode());
                    invoiceInfo.preUpdate();
                    updateInvoiceData.add(invoiceInfo);
                }
                mapper.updateInvoiceList(updateInvoiceData);
            }
            reimbursement.preUpdate();
            mapper.saveBusiness(reimbursement);
        }
    }
    @Override
    public Map<String,Object> saveGatheringTime(ReimbursementDTO reimbursement) {
        Map<String,Object> map = new HashMap<String,Object>();
        String[] idArray =reimbursement.getId().split(",");
        ArrayList<String> idList = Lists.newArrayList(idArray);
        //根据id查询发票是否已经填写过收款时间信息
        List<ReimbursementDTO> invoiceList =  mapper.getGatheringTimeInvoiceList(idList);
        if(invoiceList.size()>0){
            List<String> invoiceNumber = Lists.newArrayList();
            for (ReimbursementDTO invoice : invoiceList) {
                invoiceNumber.add(invoice.getInvoiceNumber());
            }
            String invoiceNumberStr = StringUtils.join(invoiceNumber, ",");
            map.put("message","发票号：" + invoiceNumberStr + " 已经填写过收款日期，无法重复填写。<br/> 本次收款操作失败");
            map.put("success",false);
            return map;
        }
        //修改发票收款时间
        reimbursement.preUpdate();
        mapper.updateGatheringTime(idList,reimbursement);

        for (String id : idList) {
            reimbursement.setBusinessCode(id);

            //根据该条发票信息查询同业务编号下的没有收款的发票信息
            List<ReimbursementDTO> notGatheringTimeInvoice = mapper.getNotGatheringTimeInvoice(reimbursement);
            if(notGatheringTimeInvoice.size() == 0){
                //查询该发票信息
                ReimbursementDTO reimbursementDTO = mapper.queryInvoiceById(id);
                //如果所有的发票均已收款，则将业务信息添加全部收款状态
                if(null != reimbursementDTO){
                    mapper.updateGatheringStatusById(reimbursementDTO.getParentId());
                }
            }
        }


        map.put("success",true);
        map.put("message","收款成功");
        return map;
    }

    @Override
    public void saveInvoice(ReimbursementDTO reimbursement) {
        reimbursement.preUpdate();
        mapper.saveInvoice(reimbursement);
        //根据该条发票信息查询同业务编号下的没有收款的发票信息
        List<ReimbursementDTO> notGatheringTimeInvoice = mapper.getNotGatheringTimeInvoice(reimbursement);
        if(notGatheringTimeInvoice.size() == 0){
            //查询该发票信息
            ReimbursementDTO reimbursementDTO = mapper.queryInvoiceById(reimbursement.getId());
            //如果所有的发票均已收款，则将业务信息添加全部收款状态
            mapper.updateGatheringStatusById(reimbursementDTO.getParentId());
        }
    }

    @Override
    public void modifyInvoice(ReimbursementDTO reimbursement) {
        reimbursement.preUpdate();
        mapper.modifyInvoice(reimbursement);
    }

    @Override
    public void removeByIds(List<String> idList) {
        //删除业务信息
        mapper.removeBusinessByIds(idList);
        //删除发票信息
        mapper.removeInvoiceByIds(idList);
    }

    @Override
    public void removeBusinessByIds(List<String> idList) {
        //删除业务信息
        mapper.removeBusinessByIds(idList);
        //删除发票信息
        mapper.removeBusinessByParentIds(idList);

    }

    @Override
    public void removeInvoiceByIds(List<String> idList) {
        //删除发票信息
        mapper.removeInvoiceByIds(idList);
    }

    @Override
    public void updateReimbursementRatio(ReimbursementDTO reimbursement) {
        mapper.updateReimbursementRatio(reimbursement);
    }

    /**
     * 文件数据分组
     * @param dataList
     * @return
     */
    public Map<String, List<ReimbursementDTO>> dataDisposeOnSaveByBusinessCode(List<ReimbursementDTO> dataList) {
        Map<String, List<ReimbursementDTO>> result = listMapOnSaveByBusinessCode(dataList);
        return result;
    }

    private Map<String, List<ReimbursementDTO>> listMapOnSaveByBusinessCode(List<ReimbursementDTO> list) {
        Map<String, List<ReimbursementDTO>> userSignSerialMap = new HashMap<>();
        if (CollectionUtils.isNotEmpty(list)) {
            // 分组
            for (ReimbursementDTO vo : list) {
                List<ReimbursementDTO> tempList = userSignSerialMap.get(vo.getBusinessCode());
                /*如果取不到数据,那么直接new一个空的ArrayList**/
                if (tempList == null) {
                    tempList = new ArrayList<>();
                    tempList.add(vo);
                    userSignSerialMap.put(vo.getBusinessCode(), tempList);
                }
                else {
                    /*某个vo之前已经存放过了,则直接追加数据到原来的List里**/
                    tempList.add(vo);
                }
            }
        }
        return userSignSerialMap;
    }

}
