package com.jeeplus.core.domain;

import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableLogic;
import com.jeeplus.sys.service.dto.UserDTO;
import com.jeeplus.sys.utils.UserUtils;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

import java.io.Serializable;
import java.util.Date;
import java.util.UUID;

/*
 * 数据Entity类
 * @author jeeplus
 * @version 2021-05-16
 */
@Data
public abstract class BaseEntity implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 实体主键
     */
    @TableId
    private String id;

    /**
     * 创建日期
     */
    @TableField(fill = FieldFill.INSERT)
    private Date createDate;

    /**
     * 创建人
     */
    @TableField(fill = FieldFill.INSERT)
    private String createBy;

    /**
     * 更新日期
     */
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private Date updateDate;

    /**
     * 更新人
     */
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private String updateBy;

    /**
     * 逻辑删除标记
     */
    @TableLogic
    @TableField(fill = FieldFill.INSERT)
    private Integer delFlag;


    /**
     * 默认构造函数
     */

    public BaseEntity () {

    }

    /**
     * 构造函数
     */
    public BaseEntity (String id) {
        this.id = id;
    }


    /**
     * 删除标记（0：正常；1：删除；2：审核；）
     */
    public static final String DEL_FLAG_NORMAL = "0";
    public static final String DEL_FLAG_DELETE = "1";
    public static final String DEL_FLAG_AUDIT = "2";


    /**
     * 插入之前执行方法，需要手动调用
     */
    public void preInsert(){
        setId(UUID.randomUUID().toString().replaceAll("-", ""));
        UserDTO user = UserUtils.getCurrentUserDTO();
        if (StringUtils.isNotBlank(user.getId())){
            this.setCreateBy(user.getId());
            this.setUpdateBy(user.getId());
        }
        this.setCreateDate(new Date());
        this.setUpdateDate(new Date());
        this.setDelFlag(0);
    }

    /**
     * 更新之前执行方法，需要手动调用
     */
    public void preUpdate(){
        UserDTO user = UserUtils.getCurrentUserDTO();
        if (StringUtils.isNotBlank(user.getId())){
            this.setUpdateBy(user.getId());
        }
        this.setUpdateDate(new Date());
    }
}
